/******************************************************************************
 GAddressBookTreeDir.cc

	<Description>

	BASE CLASS = public JXWindowDirector

	Copyright  1998 by Glenn W. Bach.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

// includes

//Class Header
#include <GAddressBookTreeDir.h>
#include <JTree.h>
#include <JNamedTreeNode.h>
#include <GAddressBookTreeWidget.h>
#include <JNamedTreeList.h>
#include <JXNamedTreeListWidget.h>
#include <GMGlobals.h>
#include <GPrefsMgr.h>
#include <GAddressBookMgr.h>
#include "GHelpText.h"
#include <GMApp.h>

#include <JChooseSaveFile.h>
#include <JXWindow.h>
#include <JXTextMenu.h>
#include <JXMenuBar.h>
#include <JXScrollbarSet.h>
#include <JXApplication.h>
#include <JXTextButton.h>
#include <JXTimerTask.h>
#include <JXHelpManager.h>

#include <JString.h>

#include <jAssert.h>

static const JCharacter* kHelpMenuTitleStr = "Help";
static const JCharacter* kHelpMenuStr = 
	"About"
	"%l|Table of Contents|Overview|This window"
	"%l|Changes|Credits";

enum
{	
	kAboutCmd = 1,
	kTOCCmd,
	kOverviewCmd,
	kThisWindowCmd,
	kChangesCmd,
	kCreditsCmd
};

const JCoordinate kCloseButtonWidth 	= 50;
const JCoordinate kCurrentPrefsVersion	= 0;

const Time kDirUpdateDelay = 1000;	// milliseconds

/******************************************************************************
 Constructor

 *****************************************************************************/

GAddressBookTreeDir::GAddressBookTreeDir
	(
	JXDirector* supervisor
	)
	:
   JXWindowDirector(supervisor),
   JPrefObject(GGetPrefsMgr(), kAddressTreePrefsID),
   itsPrefsOK(kFalse)
{
	BuildWindow();
	JPrefObject::ReadPrefs();
	itsTimerTask = new JXTimerTask(kDirUpdateDelay);
	assert(itsTimerTask != NULL);
	JXGetApplication()->InstallIdleTask(itsTimerTask);
	ListenTo(itsTimerTask);
	itsWidget->Update();
	if (!itsPrefsOK)
		{
		GGetAddressBookMgr()->AddDefaultAddressBook(itsTree);
		}
}

/******************************************************************************
 Destructor

 *****************************************************************************/

GAddressBookTreeDir::~GAddressBookTreeDir()
{
	JPrefObject::WritePrefs();
	itsWidget->Save();
	delete itsTree;
	delete itsTimerTask;
}

/******************************************************************************
 BuildWindow 


 ******************************************************************************/

void
GAddressBookTreeDir::BuildWindow()
{
	JCoordinate w = 300;
	JCoordinate h = 500;
	JXWindow* window = new JXWindow(this, w,h, "Addresses");
    assert( window != NULL );
    SetWindow(window);
	window->SetCloseAction(JXWindow::kDeactivateDirector);
    window->PlaceAsDialogWindow();
    
	JXMenuBar* menuBar =
		new JXMenuBar(window, JXWidget::kHElastic, JXWidget::kFixedTop, 
			0,0, w - kCloseButtonWidth,kJXStdMenuBarHeight);
	assert( menuBar != NULL );

    itsCloseButton =
        new JXTextButton("Close", window,
                    JXWidget::kFixedRight, JXWidget::kFixedTop, 
                    w - kCloseButtonWidth,0, 
                    kCloseButtonWidth,kJXStdMenuBarHeight);
    assert( itsCloseButton != NULL );
    itsCloseButton->SetShortcuts("#W"); //^[
    ListenTo(itsCloseButton);

	h -= kJXStdMenuBarHeight;
	
	JXScrollbarSet* scrollbarSet =
		new JXScrollbarSet(window, JXWidget::kHElastic,JXWidget::kVElastic,
						   0, kJXStdMenuBarHeight,
						   w, h);
	assert( scrollbarSet != NULL );
    
    itsTree = new JTree();
    assert(itsTree != NULL);
    JNamedTreeNode* base = new JNamedTreeNode(itsTree, NULL, "");
    itsTree->SetBase(base);
    JNamedTreeList* list = new JNamedTreeList(itsTree);
    
    itsWidget = 
    	new GAddressBookTreeWidget(itsTree, list,  menuBar,
    		scrollbarSet, scrollbarSet->GetScrollEnclosure(),
    		JXWidget::kHElastic, JXWidget::kVElastic,
    		0,0,w,h);
    assert(itsWidget != NULL);

	itsHelpMenu = menuBar->AppendTextMenu(kHelpMenuTitleStr);
	itsHelpMenu->SetMenuItems(kHelpMenuStr);
	itsHelpMenu->SetUpdateAction(JXMenu::kDisableNone);
	ListenTo(itsHelpMenu);
    
}

/******************************************************************************
 Receive (virtual protected)


 ******************************************************************************/

void
GAddressBookTreeDir::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsCloseButton && message.Is(JXButton::kPushed))
		{
		Deactivate();
		}
	else if (sender == itsTimerTask && message.Is(JXTimerTask::kTimerWentOff))
		{
		itsWidget->Update();
		}
	else if (sender == itsHelpMenu && message.Is(JXMenu::kItemSelected))
		{
		 const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		HandleHelpMenu(selection->GetIndex());
		}
	else
		{
		JXWindowDirector::Receive(sender, message);
		}
}

/******************************************************************************
 ReadPrefs (protected)

 ******************************************************************************/

void
GAddressBookTreeDir::ReadPrefs
	(
	istream& input
	)
{
	itsPrefsOK = kTrue;
	JCoordinate id;
	input >> id;
	assert(id <= kCurrentPrefsVersion);
	GetWindow()->ReadGeometry(input);
	itsWidget->ReadPrefs(input);
	JBoolean active;
	input >> active;
	JBoolean iconified;
	input >> iconified;
	if (iconified)
		{
		GetWindow()->Deiconify();
		}
	if (active && !iconified)
		{
		Activate();
		}
}

/******************************************************************************
 WritePrefs (protected)

 ******************************************************************************/

void
GAddressBookTreeDir::WritePrefs
	(
	ostream& output
	)
	const
{
	output << kCurrentPrefsVersion << ' ';
	GetWindow()->WriteGeometry(output);
	itsWidget->WritePrefs(output);
	output << IsActive() << ' ';
	output << GetWindow()->IsIconified() << ' ';
}

/******************************************************************************
 SaveOnDeath (public)

 ******************************************************************************/

void
GAddressBookTreeDir::SaveOnDeath()
{
	itsWidget->Save();
}

/******************************************************************************
 HandleHelpMenu 


 ******************************************************************************/

void
GAddressBookTreeDir::HandleHelpMenu
	(
	const JIndex index
	)
{
	if (index == kAboutCmd)
		{
		GMGetApplication()->DisplayAbout();
		}
	else if (index == kTOCCmd)
		{
		(JXGetHelpManager())->ShowSection(kTOCHelpName);
		}
	else if (index == kOverviewCmd)
		{
		(JXGetHelpManager())->ShowSection(kOverviewHelpName);
		}
	else if (index == kThisWindowCmd)
		{
		(JXGetHelpManager())->ShowSection(kABHelpName);
		}
	else if (index == kChangesCmd)
		{
		(JXGetHelpManager())->ShowSection(kChangeLogName);
		}
	else if (index == kCreditsCmd)
		{
		(JXGetHelpManager())->ShowSection(kCreditsName);
		}

}
