/******************************************************************************
 GAddressItemTreeNode.cc

	Data structure for JXFileTree.

	Copyright <A9> 1997 by Glenn W. Bach. All rights reserved.

 *****************************************************************************/

#include "GAddressItemTreeNode.h"
#include "GAddressEntryTreeNode.h"

#include <JString.h>

#include <jGlobals.h>

#include <jAssert.h>


/******************************************************************************
	Constructor

******************************************************************************/

GAddressItemTreeNode::GAddressItemTreeNode
	(
	JTree*	 		tree,
	Type 			type,
	JNamedTreeNode* parent,
	const char* 	name,
	const JBoolean 	isBranch
	)
	:
	GAddressBaseTreeNode(tree, parent, name, isBranch),
	itsType(type),
	itsText(name)
{
	if (type == kName)
		{
		SetName("Nickname");
		}
	else if (type == kFcc)
		{
		SetName("Fcc");
		if (parent != NULL)
			{
			GAddressEntryTreeNode* eparent = 
				dynamic_cast(GAddressEntryTreeNode*, parent);
			assert(eparent != NULL);
			eparent->SetHasFcc(kTrue);
			}
		}
	else if (type == kComment)
		{
		SetName("Comment");
		if (parent != NULL)
			{
			GAddressEntryTreeNode* eparent = 
				dynamic_cast(GAddressEntryTreeNode*, parent);
			assert(eparent != NULL);
			eparent->SetHasComment(kTrue);
			}
		}
	else if (type == kEMail)
		{
		SetName("E-mail");
		}
}


/******************************************************************************
	Destructor

******************************************************************************/

GAddressItemTreeNode::~GAddressItemTreeNode()
{
	JTreeNode* jparent = GetParent();
	if (jparent != NULL)
		{
		GAddressEntryTreeNode* parent = 
			dynamic_cast(GAddressEntryTreeNode*, jparent);
		assert(parent != NULL);
		if (itsType == kFcc)
			{
			parent->SetHasFcc(kFalse);
			}
		else if (itsType == kComment)
			{
			parent->SetHasComment(kFalse);
			}
		}
}

/******************************************************************************
 GetFullPathAndName

 ******************************************************************************/

JString
GAddressItemTreeNode::GetFullPathAndName()
	const
{
	return GetName();
}

/******************************************************************************
 SetText (public)

 ******************************************************************************/

void
GAddressItemTreeNode::SetText
	(
	const JString& text
	)
{
	itsText = text;
}

/******************************************************************************
 OKToChange

 ******************************************************************************/

JBoolean 
GAddressItemTreeNode::OKToChange()
	const
{
	JTreeNode* parent = GetParent();
	assert(parent != NULL);
	GAddressBaseTreeNode* node = dynamic_cast(GAddressBaseTreeNode*, parent);
	assert(node != NULL);
	return node->OKToChange();
}

/******************************************************************************
 Save (public)

 ******************************************************************************/

void	
GAddressItemTreeNode::Save
	(
	ostream& os
	)
{

}
