/******************************************************************************
 GMailPrefsDialog.cc

	BASE CLASS = JXDialogDirector

	Copyright @ 1997 by Glenn W. Bach.

 ******************************************************************************/

#include "GMailPrefsDialog.h"

#include <JXWindow.h>
#include <JXTextButton.h>
#include <JXInputField.h>
#include <JXTextCheckbox.h>
#include <JXStaticText.h>
#include <JXIntegerInput.h>
#include <JXChooseSaveFile.h>

#include <JColormap.h>
#include <JString.h>

#include <jXGlobals.h>

#include <jAssert.h>

/******************************************************************************
 Constructor

 ******************************************************************************/

GMailPrefsDialog::GMailPrefsDialog
	(
	JXDirector* 	supervisor,
	const JString& 	inbox,
	const JString& 	smtp,
	const JString& 	smtpuser,
	const JString& 	repltyto,
	const JBoolean 	showstate,
	const JString& 	pop,
	const JString& 	account,
	const JBoolean 	usePop,
	const JBoolean 	leave,
	const JBoolean 	save,
	const JBoolean 	useAPop,
	const JBoolean 	autoQuote,
	const JBoolean 	autoCheck,
	const JInteger	minutes,
	const JBoolean 	showSender,
	const JString& 	senderString
	)
	:
	JXDialogDirector(supervisor, kTrue)
{
	BuildWindow(inbox, smtp, smtpuser, repltyto, showstate, pop, account, usePop, leave, 
					save, useAPop, autoQuote, autoCheck, minutes, showSender,
					senderString);
}

/******************************************************************************
 Destructor

 ******************************************************************************/

GMailPrefsDialog::~GMailPrefsDialog()
{
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
GMailPrefsDialog::BuildWindow
	(
	const JString& 	inbox,
	const JString& 	smtp,
	const JString& 	smtpuser,
	const JString& 	replyto,
	const JBoolean 	showstate,
	const JString& 	pop,
	const JString& 	account,
	const JBoolean 	usePop,
	const JBoolean 	leave,
	const JBoolean 	save,
	const JBoolean 	useAPop,
	const JBoolean 	autoQuote,
	const JBoolean 	autoCheck,
	const JInteger	minutes,
	const JBoolean 	showSender,
	const JString& 	senderString
	)
{
	
// begin JXLayout

    JXWindow* window = new JXWindow(this, 370,550, "");
    assert( window != NULL );
    SetWindow(window);

    itsChooseInboxButton =
        new JXTextButton("Choose inbox", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,40, 100,20);
    assert( itsChooseInboxButton != NULL );

    JXStaticText* obj1 =
        new JXStaticText("Default inbox:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,18, 110,20);
    assert( obj1 != NULL );

    itsSMTPServerInput =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,70, 230,20);
    assert( itsSMTPServerInput != NULL );

    JXStaticText* obj2 =
        new JXStaticText("SMTP server:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,73, 90,20);
    assert( obj2 != NULL );

    itsPopServerInput =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,455, 230,20);
    assert( itsPopServerInput != NULL );

    itsPOPText3 =
        new JXStaticText("POP server:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,458, 85,20);
    assert( itsPOPText3 != NULL );

    itsUsePopButton =
        new JXTextCheckbox("Use POP server", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,285, 120,20);
    assert( itsUsePopButton != NULL );

    itsLeaveMailButton =
        new JXTextCheckbox("Leave mail on server", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 30,310, 150,20);
    assert( itsLeaveMailButton != NULL );

    itsSavePasswdButton =
        new JXTextCheckbox("Save password", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 30,335, 120,20);
    assert( itsSavePasswdButton != NULL );

    itsPopAccountInput =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,485, 230,20);
    assert( itsPopAccountInput != NULL );

    itsPOPText4 =
        new JXStaticText("POP account:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,488, 90,20);
    assert( itsPOPText4 != NULL );

    JXTextButton* okButton =
        new JXTextButton("OK", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 90,523, 70,20);
    assert( okButton != NULL );
    okButton->SetShortcuts("^M");

    JXTextButton* cancelButton =
        new JXTextButton("Cancel", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 200,523, 70,20);
    assert( cancelButton != NULL );
    cancelButton->SetShortcuts("#W^[");

    itsInboxText =
        new JXStaticText("", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,40, 230,20);
    assert( itsInboxText != NULL );

    itsShowStateInTitle =
        new JXTextCheckbox("Show mailbox state in title", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,165, 175,20);
    assert( itsShowStateInTitle != NULL );

    itsAutoQuote =
        new JXTextCheckbox("Automatically insert quote on reply", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,195, 230,20);
    assert( itsAutoQuote != NULL );

    itsUseAPop =
        new JXTextCheckbox("Use APOP", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 30,360, 120,20);
    assert( itsUseAPop != NULL );

    itsAutoCheckMail =
        new JXTextCheckbox("Automatically check mail", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 30,385, 180,20);
    assert( itsAutoCheckMail != NULL );

    itsCheckMailDelay =
        new JXIntegerInput(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,415, 70,20);
    assert( itsCheckMailDelay != NULL );
    itsCheckMailDelay->SetFontSize(10);

    itsPOPText1 =
        new JXStaticText("every", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 70,418, 45,20);
    assert( itsPOPText1 != NULL );

    itsPOPText2 =
        new JXStaticText("minutes", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 200,418, 55,20);
    assert( itsPOPText2 != NULL );

    itsSMTPUserName =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,100, 230,20);
    assert( itsSMTPUserName != NULL );

    JXStaticText* obj3 =
        new JXStaticText("SMTP user:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,103, 90,20);
    assert( obj3 != NULL );

    itsReplyToAddress =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,130, 230,20);
    assert( itsReplyToAddress != NULL );

    JXStaticText* obj4 =
        new JXStaticText("Reply to:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,133, 90,20);
    assert( obj4 != NULL );

    itsShowSenderString =
        new JXTextCheckbox("Show sender string on reply", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 20,225, 230,20);
    assert( itsShowSenderString != NULL );

    itsSenderStringInput =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 120,255, 230,20);
    assert( itsSenderStringInput != NULL );

    itsSenderText =
        new JXStaticText("Sender string:", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 30,258, 85,20);
    assert( itsSenderText != NULL );

// end JXLayout

	window->SetTitle("Mail Preferences");
	SetButtons(okButton, cancelButton);

	itsInboxText->SetBorderWidth(2);

	ListenTo(itsChooseInboxButton);
	ListenTo(itsUsePopButton);
	ListenTo(itsShowSenderString);

	itsInboxText->SetText(inbox);
	itsSMTPServerInput->SetText(smtp);
	itsSMTPUserName->SetText(smtpuser);
	itsReplyToAddress->SetText(replyto);
	itsPopServerInput->SetText(pop);
	itsPopAccountInput->SetText(account);

	itsShowStateInTitle->SetState(showstate);
	itsUsePopButton->SetState(usePop);
	itsLeaveMailButton->SetState(leave);
	itsSavePasswdButton->SetState(save);
	itsUseAPop->SetState(useAPop);

	itsAutoQuote->SetState(autoQuote);
	itsAutoCheckMail->SetState(autoCheck);
	itsCheckMailDelay->SetValue(minutes);

	itsSenderStringInput->SetText(senderString);
	itsShowSenderString->SetState(showSender);
	AdjustForPOP(usePop);
	AdjustForSenderString(showSender);

	// ******* temporary until pop working *******

//	itsPopServerInput->Deactivate();
//	itsLeaveMailButton->Deactivate();
//	itsSavePasswdButton->Deactivate();
//	itsPopAccountInput->Deactivate();
//	itsUsePopButton->Deactivate();
}

/******************************************************************************
 GetValues 

 ******************************************************************************/

void
GMailPrefsDialog::GetValues
	(
	JString* 	inbox,
	JString* 	smtp,
	JString* 	smtpuser,
	JString*	replyto,
	JBoolean* 	showstate,
	JString* 	pop,
	JString* 	account,
	JBoolean* 	usePop,
	JBoolean* 	leave,
	JBoolean* 	save,
	JBoolean* 	useAPop,
	JBoolean* 	autoQuote,
	JBoolean* 	autoCheck,
	JInteger*	minutes,
	JBoolean*	showSender,
	JString*	senderString
	)
{
	*inbox 		= itsInboxText->GetText();
	*smtp 		= itsSMTPServerInput->GetText();
	*smtpuser 	= itsSMTPUserName->GetText();
	*replyto 	= itsReplyToAddress->GetText();
	*showstate 	= itsShowStateInTitle->IsChecked();
	*pop 		= itsPopServerInput->GetText();
	*account 	= itsPopAccountInput->GetText();

	*usePop 	= itsUsePopButton->IsChecked();
	*leave 		= itsLeaveMailButton->IsChecked();
	*save 		= itsSavePasswdButton->IsChecked();

	*useAPop 	= itsUseAPop->IsChecked();
	*autoQuote 	= itsAutoQuote->IsChecked();

	*autoCheck	= itsAutoCheckMail->IsChecked();
	itsCheckMailDelay->GetValue(minutes);

	*showSender = itsShowSenderString->IsChecked();
	*senderString = itsSenderStringInput->GetText();
}

/******************************************************************************
 Receive 

 ******************************************************************************/

void
GMailPrefsDialog::Receive
	(
	JBroadcaster* 	sender, 
	const Message& 	message
	)
{
	if (sender == itsChooseInboxButton && message.Is(JXButton::kPushed))
		{
		JString mbox;
		if (JXGetChooseSaveFile()->ChooseFile("Choose default inbox:", "", &mbox))
			{
			itsInboxText->SetText(mbox);
			}
		}
	else if (sender == itsUsePopButton && message.Is(JXCheckbox::kPushed))
		{
		const JXCheckbox::Pushed* info = 
			dynamic_cast(const JXCheckbox::Pushed*, &message);
		assert(info != NULL);
		AdjustForPOP(info->IsChecked());
		}
	else if (sender == itsShowSenderString && message.Is(JXCheckbox::kPushed))
		{
		const JXCheckbox::Pushed* info = 
			dynamic_cast(const JXCheckbox::Pushed*, &message);
		assert(info != NULL);
		AdjustForSenderString(info->IsChecked());
		}
	else
		{
		JXDialogDirector::Receive(sender, message);
		}
}

/******************************************************************************
 AdjustForPOP (private)

 ******************************************************************************/

void
GMailPrefsDialog::AdjustForPOP
	(
	const JBoolean usePOP
	)
{
	JFontStyle style(kFalse, kFalse, 0, kFalse, JGetCurrColormap()->GetInactiveLabelColor());
	if (usePOP)
		{
		itsPopServerInput->Activate();
		itsLeaveMailButton->Activate();
		itsSavePasswdButton->Activate();
		itsPopAccountInput->Activate();
		itsUseAPop->Activate();
		itsAutoCheckMail->Activate();
		itsCheckMailDelay->Activate();
		itsPOPText1->SetFontStyle(JFontStyle());
		itsPOPText2->SetFontStyle(JFontStyle());
		itsPOPText3->SetFontStyle(JFontStyle());
		itsPOPText4->SetFontStyle(JFontStyle());
		itsCheckMailDelay->SetFontStyle(JFontStyle());
		itsPopAccountInput->SetFontStyle(JFontStyle());
		itsPopServerInput->SetFontStyle(JFontStyle());
		}
	else
		{
		itsPopServerInput->Deactivate();
		itsLeaveMailButton->Deactivate();
		itsSavePasswdButton->Deactivate();
		itsPopAccountInput->Deactivate();
		itsUseAPop->Deactivate();
		itsAutoCheckMail->Deactivate();
		itsCheckMailDelay->Deactivate();
		itsPOPText1->SetFontStyle(style);
		itsPOPText2->SetFontStyle(style);
		itsPOPText3->SetFontStyle(style);
		itsPOPText4->SetFontStyle(style);
		itsCheckMailDelay->SetFontStyle(style);
		itsPopAccountInput->SetFontStyle(style);
		itsPopServerInput->SetFontStyle(style);
		}
}

/******************************************************************************
 AdjustForSenderString (private)

 ******************************************************************************/

void
GMailPrefsDialog::AdjustForSenderString
	(
	const JBoolean showSender
	)
{
	JFontStyle style(kFalse, kFalse, 0, kFalse, JGetCurrColormap()->GetInactiveLabelColor());
	if (showSender)
		{
		itsSenderStringInput->Activate();
		itsSenderText->SetFontStyle(JFontStyle());
		itsSenderStringInput->SetFontStyle(JFontStyle());
		}
	else
		{
		itsSenderStringInput->Deactivate();
		itsSenderText->SetFontStyle(style);
		itsSenderStringInput->SetFontStyle(style);
		}
}
