/******************************************************************************
 GMessageHeader.cc

	BASE CLASS = <NONE>

	Copyright  1997 by Glenn W. Bach.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

//Class Header
#include <GMessageHeader.h>

#include <JRegex.h>
#include <JPtrArray.h>

#include <gMailUtils.h>

#include <jAssert.h>

/******************************************************************************
 Constructor

 *****************************************************************************/

GMessageHeader::GMessageHeader()
{
	itsMessageStatus = kNew;
}

/******************************************************************************
 Destructor

 *****************************************************************************/

GMessageHeader::~GMessageHeader()
{
}

/******************************************************************************
 SetHeader

 *****************************************************************************/

void
GMessageHeader::SetHeader
	(
	const JString& header
	)
{
	itsHeader = header;
	itsHeader.TrimWhitespace();
	JRegex regex;
	JArray<JIndexRange>* subList = new JArray<JIndexRange>;
	assert(subList != NULL);
	JError err = regex.SetPattern("[[:space:]]+([0-9]+)$");
	assert(err.OK());
	JBoolean matched = regex.Match(itsHeader, subList);
	if (matched)
		{
		JIndexRange sRange = subList->GetElement(2);
		JString year = itsHeader.GetSubstring(sRange);
		year.ConvertToUInt(&itsYear);
		}

	err = regex.SetPattern("[0-9][0-9]:[0-9][0-9]:[0-9][0-9]");
	assert(err.OK());
	matched = regex.Match(itsHeader, subList);
	if (matched)
		{
		JIndexRange sRange = subList->GetElement(1);
		itsTime = itsHeader.GetSubstring(sRange);
		}
	else
		{
		err = regex.SetPattern("[0-9][0-9]:[0-9][0-9]");
		assert(err.OK());
		matched = regex.Match(itsHeader, subList);
		if (matched)
			{
			JIndexRange sRange = subList->GetElement(1);
			itsTime = itsHeader.GetSubstring(sRange);
			}
		}
}

/******************************************************************************
 SetFrom

 *****************************************************************************/

void
GMessageHeader::SetFrom
	(
	const JString& from
	)
{
	itsFrom = from;
	JPtrArray<JString> names;
	GGetRealNames(from, names);
	if (names.GetElementCount() > 0)
		{
		itsFromNames = *(names.NthElement(1));
		for (JSize i = 2; i <= names.GetElementCount(); i++)
			{
			itsFromNames += ", " + *(names.NthElement(i));
			}
		}
	else
		{
		itsFromNames = from;
		}
	
}

/******************************************************************************
 SetAddress

 *****************************************************************************/

void
GMessageHeader::SetAddress
	(
	const JString& address
	)
{
	itsAddress = address;
}

/******************************************************************************
 SetSubject

 *****************************************************************************/

void
GMessageHeader::SetSubject
	(
	const JString& subject
	)
{
	itsSubject = subject;
	itsSubject.TrimWhitespace();
}

/******************************************************************************
 SetDate

 *****************************************************************************/

void
GMessageHeader::SetDate
	(
	const JString& date
	)
{
	itsDate = date;
	itsDate.TrimWhitespace();
	JRegex regex;
	JArray<JIndexRange>* subList = new JArray<JIndexRange>;
	assert(subList != NULL);
	JError err = regex.SetPattern("^[a-zA-Z]+,[[:space:]]+([0-9]+)[[:space:]]+([a-zA-Z]+)");
	assert(err.OK());
	JBoolean matched = regex.Match(itsDate, subList);
	if (matched)
		{
		JIndexRange sRange = subList->GetElement(3);
		itsMonth = itsDate.GetSubstring(sRange);
		sRange = subList->GetElement(2);
		JString day = itsDate.GetSubstring(sRange);
		if (!day.IsEmpty())
			{
			if (day.GetFirstCharacter() == '0')
				{
				day.RemoveSubstring(1,1);
				}
			}
		day.ConvertToUInt(&itsDay);
		itsShortDate = itsMonth + " " + day; 
		return;
		}
	err = regex.SetPattern("^([0-9]+)[[:space:]]+([a-zA-Z]+)");
	assert(err.OK());
	matched = regex.Match(itsDate, subList);
	if (matched)
		{
		JIndexRange sRange = subList->GetElement(3);
		itsMonth = itsDate.GetSubstring(sRange);
		sRange = subList->GetElement(2);
		JString day = itsDate.GetSubstring(sRange);
		if (!day.IsEmpty())
			{
			if (day.GetFirstCharacter() == '0')
				{
				day.RemoveSubstring(1,1);
				}
			}
		day.ConvertToUInt(&itsDay);
		itsShortDate = itsMonth + " " + day; 
		return;
		}
	else
		{
		itsShortDate = itsDate;
		}
}

/******************************************************************************
 SetReplyTo

 *****************************************************************************/

void
GMessageHeader::SetReplyTo
	(
	const JString& replyto
	)
{
	itsReplyTo = replyto;
}

/******************************************************************************
 SetCC

 *****************************************************************************/

void
GMessageHeader::SetCC
	(
	const JString& cc
	)
{
	itsCC = cc;
}

/******************************************************************************
 SetTo

 *****************************************************************************/

void
GMessageHeader::SetTo
	(
	const JString& to
	)
{
	itsTo = to;
}

/******************************************************************************
 SetHeaderStart

 *****************************************************************************/

void
GMessageHeader::SetHeaderStart
	(
	const JIndex start
	)
{
	itsHeaderStart = start;
}

/******************************************************************************
 SetHeaderEnd

 *****************************************************************************/

void
GMessageHeader::SetHeaderEnd
	(
	const JIndex end
	)
{
	itsHeaderEnd = end;
}

/******************************************************************************
 SetMessageEnd

 *****************************************************************************/

void
GMessageHeader::SetMessageEnd
	(
	const JIndex end
	)
{
	itsMessageEnd = end;
}

/******************************************************************************
 SetMessageStatus

 *****************************************************************************/

void
GMessageHeader::SetMessageStatus
	(
	const MessageStatus status
	)
{
	itsMessageStatus = status;
}

/******************************************************************************
 GetHeader

 *****************************************************************************/

const JString&
GMessageHeader::GetHeader()
	const
{
	return itsHeader;
}

/******************************************************************************
 GetFrom

 *****************************************************************************/

const JString&
GMessageHeader::GetFrom()
	const
{
	return itsFrom;
}

/******************************************************************************
 GetFromNames

 *****************************************************************************/

const JString&
GMessageHeader::GetFromNames()
	const
{
	return itsFromNames;
}

/******************************************************************************
 GetTo

 *****************************************************************************/

const JString&
GMessageHeader::GetTo()
	const
{
	return itsTo;
}

/******************************************************************************
 GetAddress

 *****************************************************************************/

const JString&
GMessageHeader::GetAddress()
	const
{
	return itsAddress;
}


/******************************************************************************
 GetSubject

 *****************************************************************************/

const JString&
GMessageHeader::GetSubject()
	const
{
	return itsSubject;
}

/******************************************************************************
 GetDate

 *****************************************************************************/

const JString&
GMessageHeader::GetDate()
	const
{
	return itsDate;
}

/******************************************************************************
 GetShortDate

 *****************************************************************************/

const JString&
GMessageHeader::GetShortDate()
	const
{
	return itsShortDate;
}

/******************************************************************************
 GetMonth

 *****************************************************************************/

const JString&
GMessageHeader::GetMonth()
	const
{
	return itsMonth;
}

/******************************************************************************
 GetYear

 *****************************************************************************/

JIndex
GMessageHeader::GetYear()
	const
{
	return itsYear;
}

/******************************************************************************
 GetDay

 *****************************************************************************/

JIndex
GMessageHeader::GetDay()
	const
{
	return itsDay;
}

/******************************************************************************
 GetTime

 *****************************************************************************/

const JString&
GMessageHeader::GetTime()
	const
{
	return itsTime;
}

/******************************************************************************
 GetReplyTo

 *****************************************************************************/

const JString&
GMessageHeader::GetReplyTo()
	const
{
	return itsReplyTo;
}

/******************************************************************************
 GetCC

 *****************************************************************************/

const JString&
GMessageHeader::GetCC()
	const
{
	return itsCC;
}

/******************************************************************************
 GetHeaderStart

 *****************************************************************************/

JIndex
GMessageHeader::GetHeaderStart()
	const
{
	return itsHeaderStart;
}

/******************************************************************************
 GetHeaderEnd

 *****************************************************************************/

JIndex
GMessageHeader::GetHeaderEnd()
	const
{
	return itsHeaderEnd;
}

/******************************************************************************
 GetMessageEnd

 *****************************************************************************/

JIndex
GMessageHeader::GetMessageEnd()
	const
{
	return itsMessageEnd;
}

/******************************************************************************
 GetMessageStatus

 *****************************************************************************/

GMessageHeader::MessageStatus
GMessageHeader::GetMessageStatus()
	const
{
	return itsMessageStatus;
}

/******************************************************************************
 AdjustPosition

 *****************************************************************************/

void
GMessageHeader::AdjustPosition
	(
	const JIndex newposition
	)
{
	int delta = itsHeaderStart - newposition;
	itsHeaderStart = newposition;
	itsHeaderEnd -= delta;
	itsMessageEnd -= delta;
}

/******************************************************************************
 Stream operators

	The string data is delimited by double quotes:  "this is a string".

	To include double quotes in a string, use \"
	To include a backslash in a string, use \\

	An exception is made if the streams are cin or cout.
	For input, characters are simply read until 'return' is pressed.
	For output, Print() is used.

 ******************************************************************************/

istream&
operator>>
	(
	istream&	input,
	GMessageHeader&	header
	)
{
	JString temp;
	input >> temp;
	header.SetHeader(temp);
	input >> temp;
	header.SetFrom(temp);
	input >> temp;
	header.SetTo(temp);
	input >> temp;
	header.SetAddress(temp);
	input >> temp;
	header.SetSubject(temp);
	input >> temp;
	header.SetDate(temp);
	int tempi;
	input >> tempi;
	header.SetHeaderStart(tempi);
	input >> tempi;
	header.SetHeaderEnd(tempi);
	input >> tempi;
	header.SetMessageEnd(tempi);
	input >> tempi;
	header.SetMessageStatus((GMessageHeader::MessageStatus)tempi);
	return input;
}

ostream&
operator<<
	(
	ostream&		output,
	const GMessageHeader&	header
	)
{
	output << header.GetHeader();
	output << header.GetFrom();
	output << header.GetTo();
	output << header.GetAddress();
	output << header.GetSubject();
	output << header.GetDate();
	output << ' ';
	output << header.GetHeaderStart();
	output << ' ';
	output << header.GetHeaderEnd();
	output << ' ';
	output << header.GetMessageEnd();
	output << ' ';
	GMessageHeader::MessageStatus status = header.GetMessageStatus();
	if (status == GMessageHeader::kDelete)
		{
		status = GMessageHeader::kRead;
		}
	output << status;
	output << ' ';
	return output;
}
