/******************************************************************************
 JSpellChecker.cc

	<Description>

	BASE CLASS = <NONE>

	Copyright  1997 by Glenn W. Bach.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

#include <JSpellChecker.h>
#include <JXSpellCheckerDialog.h>

#include <JTextEditor.h>
#include <JProcess.h>
#include <JInPipeStream.h>
#include <JOutPipeStream.h>
#include <JString.h>
#include <JUserNotification.h>

#include <jTypes.h>
#include <jStreamUtil.h>
#include <jProcessUtil.h>
#include <jStrStreamUtil.h>
#include <jDirUtil.h>

#include <GMGlobals.h>

#include <strstream.h>
#include <iostream.h>
#include <jAssert.h>

const JCharacter* JSpellChecker::kSpellCheckFinished = "JSpellChecker::kSpellCheckFinished";

const JCharacter EOT = '\4';

/******************************************************************************
 Constructor

 *****************************************************************************/

JSpellChecker::JSpellChecker
	(
	JTextEditor* editor
	)
	:
	JBroadcaster()
{
	itsEditor 		= editor;
	itsCurrentIndex	= 1;
	itsDialog 		= NULL;
	itsWords 		= NULL;

	int toFD, fromFD;

	JString cwd = JGetCurrentDirectory();
	JString home;
	if (JGetHomeDirectory(&home))
		{
		JChangeDirectory(home);
		}

	JError err = JProcess::Create(&itsProcess, "ispell -a",
									kJCreatePipe, &toFD,
									kJCreatePipe, &fromFD,
									kJTossOutput, NULL);
/*	pid_t pid;
	JError err = JExecute("ispell -a", &pid, 
							kJCreatePipe, &toFD,
							kJCreatePipe, &fromFD,
							kJTossOutput, NULL);
*/
	assert (err.OK());

	itsIn = new JInPipeStream(fromFD, kTrue);
	assert(itsIn != NULL);
	itsOut = new JOutPipeStream(toFD, kTrue);
	assert(itsOut != NULL);

	char s[255];
	itsIn->getline(s, 255);
	JChangeDirectory(cwd);
}

/******************************************************************************
 Destructor

 *****************************************************************************/

JSpellChecker::~JSpellChecker()
{
	*itsOut << '#';
	*itsOut << endl;
	delete itsOut;
	delete itsIn;
	delete itsProcess;
}

/******************************************************************************
 Receive

 *****************************************************************************/

void
JSpellChecker::Receive
	(
	JBroadcaster* 					sender,
	const JBroadcaster::Message& 	message
	)
{
	if (sender == itsDialog && message.Is(JXDialogDirector::kDeactivated))
		{
		const JXDialogDirector::Deactivated* info =
			dynamic_cast(const JXDialogDirector::Deactivated*, &message);
		assert( info != NULL );
		if (info->Successful())
			{
			JXSpellCheckerDialog::SpellAction action = 
				itsDialog->GetSpellAction();
			if (action == JXSpellCheckerDialog::kChange)
				{
				itsEditor->Paste(itsDialog->GetCorrection());
				itsCurrentIndex = itsEditor->GetWordEnd(itsEditor->GetInsertionIndex()) + 1;
				}
			else if (action == JXSpellCheckerDialog::kLearn)
				{
				JString orig(itsDialog->GetCheckWord());
				*itsOut << '&';
				orig.Print(*itsOut);
				*itsOut << endl;
				*itsOut << '#';
				*itsOut << endl;
				}
			else if (action == JXSpellCheckerDialog::kLearnCaps)
				{
				JString orig(itsDialog->GetCheckWord());
				*itsOut << '*';
				orig.Print(*itsOut);
				*itsOut << endl;
				*itsOut << '#';
				*itsOut << endl;
				}
			else if (action == JXSpellCheckerDialog::kIgnoreAll)
				{
				JString orig(itsDialog->GetCheckWord());
				*itsOut << '@';
				orig.Print(*itsOut);
				*itsOut << endl;
				}
			else if (action == JXSpellCheckerDialog::kChangeAll)
				{
				itsEditor->Paste(itsDialog->GetCorrection());
				itsCurrentIndex = itsEditor->GetWordEnd(itsEditor->GetInsertionIndex()) + 1;
				JString orig(itsDialog->GetCheckWord());
				JBoolean toss;
				while (itsEditor->SearchForward(orig, kTrue, kTrue, kFalse, &toss))
					{
					itsEditor->Paste(itsDialog->GetCorrection());
					}
				}
			itsDialog = NULL;
			CheckWords();
			}
		else
			{
			itsEditor->SetCaretLocation(1);
			Broadcast(SpellCheckFinished());
			itsDialog = NULL;
			}
		}
}

/******************************************************************************
 CheckWords

 *****************************************************************************/

void
JSpellChecker::CheckWords()
{
	JBoolean ok = kTrue;
	while (ok)
		{
		JIndex end = itsEditor->GetWordEnd(itsCurrentIndex);
		JIndex start = itsEditor->GetWordStart(end);
		if ((end < start) || (start < itsCurrentIndex))
			{
			ok = kFalse;
			itsEditor->SetCaretLocation(1);
//			JGetUserNotification()->DisplayMessage("Spell check finished.");
			Broadcast(SpellCheckFinished());
			}
		else
			{
			itsEditor->SetSelection(start, end);
//			itsEditor->TEActivate();
			JString text;
			JBoolean selected = itsEditor->GetSelection(&text);
			assert(selected);
			ok = CheckWord(text);
			itsCurrentIndex = end + 1;
			}
		}
}

/******************************************************************************
 CheckWord

 *****************************************************************************/

JBoolean
JSpellChecker::CheckWord
	(
	const JString& word
	)
{
	if (word.IsInteger() || word.IsFloat())
		{
		return kTrue;
		}
	if (itsWords != NULL)
		{
		itsWords->DeleteAll();
		}
	word.Print(*itsOut);
	*itsOut << endl;
	JString test = JReadLine(*itsIn);
	if (test.IsEmpty())
		{
		test = JReadLine(*itsIn);
		}
//	if (itsIn->peek())
//		{
//		*itsIn >> ws;
//		}
	if (test.IsEmpty())
		{
		return kTrue;
		}
	JCharacter c = test.GetFirstCharacter();
	if ((c == '*') || (c == '+') || (c == '-'))
		{
		return kTrue;
		}
	itsEditor->TEScrollToSelection(kFalse);
	assert(itsDialog == NULL);
	itsDialog = new JXSpellCheckerDialog(JXGetApplication());
	assert(itsDialog != NULL);
	JIndex findex;
	if (test.LocateSubstring(":", &findex))
		{
		test.RemoveSubstring(1, findex);
		test.TrimWhitespace();
		if (itsWords == NULL)
			{
			itsWords = new JPtrArray<JString>;
			assert(itsWords != NULL);
			}
		while (test.LocateSubstring(",", &findex))
			{
			JString* str = new JString(test.GetSubstring(1, findex - 1));
			assert(str != NULL);
			str->TrimWhitespace();
			itsWords->Append(str);
			test.RemoveSubstring(1, findex);
			}
		JString* str = new JString(test);
		assert(str != NULL);
		str->TrimWhitespace();
		itsWords->Append(str);
		assert(itsWords->GetElementCount() > 0);
		if (c == '&')
			{
			itsDialog->SetFirstGuess(*(itsWords->NthElement(1)));
			}
		itsDialog->SetSuggestions(itsWords);
		}
	itsDialog->SetCheckWord(word);
	ListenTo(itsDialog);
	itsDialog->BeginDialog();
	return kFalse;
}